%combines timecourse data together into a single matrix

timepoints = steps; %the number of steps in the time-course analysis.
for qq=1:numsubjects
    currsubject=num2str(qq);
    
    individualfile = ['demo',currsubject,'timecourse.txt'];
    currentdata = importdata(individualfile);
    totaltimepoints = size(currentdata,1);
    currtrials=totaltimepoints/timepoints;
    
    
    for t=1:timepoints
        
        %one timepoint for all n trials
        currdata=zeros(currtrials,10);
        currdata(:,1)=currentdata(t:timepoints:totaltimepoints,1); %target fixations
        currdata(:,2)=currentdata(t:timepoints:totaltimepoints,2); %comp fixations
        currdata(:,3)=currentdata(t:timepoints:totaltimepoints,3); %o1 fixations
        currdata(:,4)=currentdata(t:timepoints:totaltimepoints,4); %o2 fixations
        currdata(:,5)=currentdata(t:timepoints:totaltimepoints,5); %order
        currdata(:,6)=currentdata(t:timepoints:totaltimepoints,6); %subject
        currdata(:,7)=currentdata(t:timepoints:totaltimepoints,7); %timesteps (10ms bins)
        currdata(:,8)=currentdata(t:timepoints:totaltimepoints,8); %animacy
        currdata(:,9)=currentdata(t:timepoints:totaltimepoints,9); %condition code
        currdata(:,10)=currentdata(t:timepoints:totaltimepoints,10); %trialID
        
        present1=zeros(currtrials,10); %competitor present, animate
        present2=zeros(currtrials,10); %competitor present, inanimate
        absent1=zeros(currtrials,10); %competitor absent, animate
        absent2=zeros(currtrials,10); %competitor absent, inanimate
        
        for n=1:currtrials
            if currdata(n,9)==1 %competitor present
                if currdata(n,8)==1 %animate
                    present1(n,1:10)=currdata(n,1:10);
                elseif currdata(n,8)==0 %inanimate
                    present2(n,1:10)=currdata(n,1:10);
                end
            elseif currdata(n,9)==0 %competitor absent
                if currdata(n,8)==1 %animate
                    absent1(n,1:10)=currdata(n,1:10);
                elseif currdata(n,8)==0 %inanimate
                    absent2(n,1:10)=currdata(n,1:10);
                end
            end
        end
        
        %Proportion of fixations to targets, competitors, other1, other2, summed across trials in each condition by subject, saved to a big matrix
        %target fixations
        fixsums(t,((qq-1)*16)+1) = (nansum(present1(:,1))/(nansum(present1(:,1))+nansum(present1(:,2))+nansum(present1(:,3))+nansum(present1(:,4)))) ; %present1 (animate)
        fixsums(t,((qq-1)*16)+2) = (nansum(present2(:,1))/(nansum(present2(:,1))+nansum(present2(:,2))+nansum(present2(:,3))+nansum(present2(:,4)))) ; %present2 (inanimate)
        fixsums(t,((qq-1)*16)+3) = (nansum(absent1(:,1))/(nansum(absent1(:,1))+nansum(absent1(:,2))+nansum(absent1(:,3))+nansum(absent1(:,4)))) ; %absent1
        fixsums(t,((qq-1)*16)+4) = (nansum(absent2(:,1))/(nansum(absent2(:,1))+nansum(absent2(:,2))+nansum(absent2(:,3))+nansum(absent2(:,4)))) ; %absent2
        %competitor fixations
        fixsums(t,((qq-1)*16)+5) = (nansum(present1(:,2))/(nansum(present1(:,1))+nansum(present1(:,2))+nansum(present1(:,3))+nansum(present1(:,4)))) ; %present1
        fixsums(t,((qq-1)*16)+6) = (nansum(present2(:,2))/(nansum(present2(:,1))+nansum(present2(:,2))+nansum(present2(:,3))+nansum(present2(:,4)))) ; %present2
        fixsums(t,((qq-1)*16)+7) = (nansum(absent1(:,2))/(nansum(absent1(:,1))+nansum(absent1(:,2))+nansum(absent1(:,3))+nansum(absent1(:,4)))) ; %absent1
        fixsums(t,((qq-1)*16)+8) = (nansum(absent2(:,2))/(nansum(absent2(:,1))+nansum(absent2(:,2))+nansum(absent2(:,3))+nansum(absent2(:,4)))) ; %absent2
        %other 1 fixations
        fixsums(t,((qq-1)*16)+9) = (nansum(present1(:,3))/(nansum(present1(:,1))+nansum(present1(:,2))+nansum(present1(:,3))+nansum(present1(:,4)))) ; %present1
        fixsums(t,((qq-1)*16)+10) = (nansum(present2(:,3))/(nansum(present2(:,1))+nansum(present2(:,2))+nansum(present2(:,3))+nansum(present2(:,4)))) ; %present2
        fixsums(t,((qq-1)*16)+11) = (nansum(absent1(:,3))/(nansum(absent1(:,1))+nansum(absent1(:,2))+nansum(absent1(:,3))+nansum(absent1(:,4)))) ; %absent1
        fixsums(t,((qq-1)*16)+12) = (nansum(absent2(:,3))/(nansum(absent2(:,1))+nansum(absent2(:,2))+nansum(absent2(:,3))+nansum(absent2(:,4)))) ; %absent2
        %other 2 fixations
        fixsums(t,((qq-1)*16)+13) = (nansum(present1(:,4))/(nansum(present1(:,1))+nansum(present1(:,2))+nansum(present1(:,3))+nansum(present1(:,4)))) ; %present1
        fixsums(t,((qq-1)*16)+14) = (nansum(present2(:,4))/(nansum(present2(:,1))+nansum(present2(:,2))+nansum(present2(:,3))+nansum(present2(:,4)))) ; %present2
        fixsums(t,((qq-1)*16)+15) = (nansum(absent1(:,4))/(nansum(absent1(:,1))+nansum(absent1(:,2))+nansum(absent1(:,3))+nansum(absent1(:,4)))) ; %absent1
        fixsums(t,((qq-1)*16)+16) = (nansum(absent2(:,4))/(nansum(absent2(:,1))+nansum(absent2(:,2))+nansum(absent2(:,3))+nansum(absent2(:,4)))) ; %absent2
        
    end % end loop that goes through timepoints
    
end % end loop that goes through all subjects
fixsums(isnan(fixsums)) = 0 ; %replace NANs with 0 to facilitate analysis.
dlmwrite('demo_timecourse.txt',fixsums,'delimiter','\t'); %One file 4 all subjects--fixations over time.
% target-p1 target-p2 target-a1 target-a2 competitor-p1 competitor-p2 competitor-a1 competitor-a2 other1-p1 other1-p2 other2-a1 other2-a2 other2-p1 other2-p2 other2-a1 other2-a2

%% Average the timecourse data for plotting 
for qq=1:numsubjects
    rows=size(fixsums,1);
    cols=size(fixsums,2);
    
    %conditions:
    cond1t=fixsums(1:rows,1:16:cols);%for rows , sum every 16th column starting with col #1
    cond2t=fixsums(1:rows,2:16:cols);%for rows , sum every 16th column starting with col #2...
    cond3t=fixsums(1:rows,3:16:cols);
    cond4t=fixsums(1:rows,4:16:cols);
    cond1c=fixsums(1:rows,5:16:cols);
    cond2c=fixsums(1:rows,6:16:cols);
    cond3c=fixsums(1:rows,7:16:cols);
    cond4c=fixsums(1:rows,8:16:cols);
    cond1o1=fixsums(1:rows,9:16:cols);
    cond2o1=fixsums(1:rows,10:16:cols);
    cond3o1=fixsums(1:rows,11:16:cols);
    cond4o1=fixsums(1:rows,12:16:cols);
    cond1o2=fixsums(1:rows,13:16:cols);
    cond2o2=fixsums(1:rows,14:16:cols);
    cond3o2=fixsums(1:rows,15:16:cols);
    cond4o2=fixsums(1:rows,16:16:cols);
    
    %average across all "subjects" in each cond
    cond1tm = mean(cond1t,2);
    cond2tm = mean(cond2t,2);
    cond3tm = mean(cond3t,2);
    cond4tm = mean(cond4t,2);
    cond1cm = mean(cond1c,2);
    cond2cm = mean(cond2c,2);
    cond3cm = mean(cond3c,2);
    cond4cm = mean(cond4c,2);
    cond1o1m = mean(cond1o1,2);
    cond2o1m = mean(cond2o1,2);
    cond3o1m = mean(cond3o1,2);
    cond4o1m = mean(cond4o1,2);
    cond1o2m = mean(cond1o2,2);
    cond2o2m = mean(cond2o2,2);
    cond3o2m = mean(cond3o2,2);
    cond4o2m = mean(cond4o2,2);
    
    %new matrix with means for 4 conditions, averaged across subjects
    TCmeans(1:rows,1)=cond1tm;
    TCmeans(1:rows,2)=cond2tm;
    TCmeans(1:rows,3)=cond3tm;
    TCmeans(1:rows,4)=cond4tm;
    TCmeans(1:rows,5)=cond1cm;
    TCmeans(1:rows,6)=cond2cm;
    TCmeans(1:rows,7)=cond3cm;
    TCmeans(1:rows,8)=cond4cm;
    TCmeans(1:rows,9)=cond1o1m;
    TCmeans(1:rows,10)=cond2o1m;
    TCmeans(1:rows,11)=cond3o1m;
    TCmeans(1:rows,12)=cond4o1m;
    TCmeans(1:rows,13)=cond1o2m;
    TCmeans(1:rows,14)=cond2o2m;
    TCmeans(1:rows,15)=cond3o2m;
    TCmeans(1:rows,16)=cond4o2m;

end
dlmwrite('demo_TCmeans.txt',TCmeans,'delimiter','\t','-append'); %One file averaged across subjects.
plot(TCmeans); %shows you a plot, over time, with a different line for each subject, the target advantage score.









