%Demo Experiment by Sarah Brown-Schmidt 8-7-2013; updated by SBS 8-18-2015

%This experiment is in two phases.
%Phase 1 trains subjects on picture names by showing a series of pictures and names below
%Phase 2 records eye-movements, voice, and clicks (with RT) as the subject produces an instruction to click on one of 4 pictures.

%Some details:
%This experiment was programmed to run on a mac. To switch to PC, you'll want to change the direction of file directories "/" to "\", and a few other changes having to do with PsychPortAudio settings.
%Target trials record EM and play sound. EM saved as S_subnum_randomnum
%Trial sequence: pictures, ETrecord, SOUNDrecord. A message is sent to the tracker immediately before sound record, allowing you to accuratley synch the eye movement record with the sound.
%At the beginning of the experiment it does a calibration check and drift-corrects between trials.

%See this webpage about the audio command we're using. You need special DLLs to make it work.
%http://psychtoolbox.org/wikka.php?wakka=PsychPortAudio
%Psychtoolbox will print to screen info about audio and video timing delays. Take note of the values & follow on-screen instructions.

%The items in test trials are saved in "items.txt"; the items on train trials are in "Training_list.txt"
%In this demonstration, I set up an experiment with two lists, that rotate items across one condition (cohort competitor present vs. absent).
%A second condition (whether the target is animate or not, is also manipulated between-items).
%You can easily change the items by editing these text files.


clear all;

testing = 1; % default is to do the training on picture names
testingoption = upper(input('Train on picture names in this session? (Y for Yes, N for No): ','s'));
if strcmp(testingoption, 'Y')
    testing = 1;
elseif strcmp(testingoption, 'N')
    testing = 0;
end

%testing=1; %set to 1 if you don't want to train on the picture names first.


%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%USER INPUT%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%

subjno = inputnumber('Please enter the subject number (value should be between 1 and 99): ',1,99); %enter a number between 1 and 99
listno = inputnumber('Please enter list number (1 or 2): ',1,2); %enter a number between 1 and 2

%Do eyetracking?
doeyetracking = -1;
while doeyetracking < 0
    userstr = upper(input('Do eye-tracking in this session? (Y for Yes, N for No): ','s'));
    if strcmp(userstr, 'Y')
        doeyetracking = 1;
    elseif strcmp(userstr, 'N')
        doeyetracking = 0;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%SETUP%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%

%Get a random number
rand('twister',sum(100*clock)) %resets the random number generator.
nums=rand*50;
sran=round(nums);
srand=num2str(sran); %gets a random integer for use in filenames

%Define Graphics Variables
white = [255 255 255];
black = [0 0 0];
green2 = [34 139 34];

textfont = 'Helvetica';
textsize = 125;

% Graphics Set-up
Screen('Preference','VBLTimestampingMode',-1); %This setting can be turned on if you have video driver problems.
[window, rect] = Screen('OpenWindow',0, white); %set up graphics (rect is the size of the window and the 0 is just to make it the window)
Screen('TextSize',window,textsize); % specify text size
Screen('TextFont',window,textfont);
Screen('TextStyle', window,0); % change default font to non-bold (necessary for windows computers)

Xleft=rect(RectLeft);
Xright=rect(RectRight);
Ytop=rect(RectTop);
Ybottom=rect(RectBottom);
Xcenter=Xright/2;
Ycenter=Ybottom/2;
Xqtr=Xright/4;
Yqtr=Ybottom/4;
X3qtr=Xright-Xqtr;
Y3qtr=Ybottom-Yqtr;

%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%BEGIN EXP: Training%%
%%%%%%%%%%%%%%%%%%%%%%%%%

Demo_Expt2_instructions1 % Instructions for part 1 (training)

if testing == 1  %set to 1 to complete training trials
    % --Training Trials--
    filename = ('lists/Training_list.txt');
    infile = fopen(filename);
    trainingitems = textscan(infile, '%s');
    TRAININGITEM = 1;
    fclose(infile);
    
    numtraining=numel(trainingitems{1});  %total number of train trials
    randtrain=randperm(numtraining); %vector with train trials in a random order
    
    %Training Trials
    spotforpicture = [Xcenter-250 Ycenter-250 Xcenter+250 Ycenter+250];
    for t = 1:numtraining %for testing
        Screen('FillRect',window,white,rect); %blank out the screen.
        
        trainingnum=randtrain(t);
        trainport=Screen(window,'OpenoffscreenWindow',white,[0 0 500 500]);
        textport=Screen(window,'OpenoffscreenWindow',white,rect);
        
        image=imread(['pictures/' trainingitems{TRAININGITEM}{trainingnum} '.jpg']); %sbs
        Screen(trainport,'PutImage',image,[0 0 500 500],('srcCopy')); %sbs
        Screen('CopyWindow', trainport,window,[0 0 500 500],spotforpicture); %copies to main window
        
        
        %critical word
        Screen('TextSize',window,24);
        Screen('DrawText', window, trainingitems{TRAININGITEM}{trainingnum} , Xcenter-10, Ycenter+400, black);
        
        %click button
        Screen('FillOval', window, green2, [rect(3)-150 rect(4)-150 rect(3)-25 rect(4)-25]); %"Click" icon
        Screen('DrawText', window, 'click' , rect(3)-138, rect(4)-115, black);
        Screen('Flip', window,  [],1);
        GetClicks;
        
    end
end

%Patience...
Screen('FillRect',window,white,rect); %blank out the screen.
Screen('DrawText', window, 'Please wait...' , Xcenter-20, Ycenter, black);
Screen('Flip', window,  [],1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%MAIN TASK: Eyetracking%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Demo_Expt2_instructions2 % Instructions for part 2 (testing)

%%%%%%%%%%%%%%%%%%%%%%%%
%%%%Eyetracker Stuff%%%%
%%%%%%%%%%%%%%%%%%%%%%%%
if doeyetracking
    
    commandwindow;
    % STEP 1
    % Initialization of the connection with the Eyelink Gazetracker.
    % exit program if this fails.
    
    if EyelinkInit()~= 1; %
        return;
    end;
    
    % STEP 2
    % Open a graphics window on the main screen
    % using the PsychToolbox's Screen function.
    if 1; Screen('Preference', 'SkipSyncTests', 1); end %The last # should be "1" for testing mode only.
    screenNumber=max(Screen('Screens'));
    [window, wRect]=Screen('OpenWindow', screenNumber, 0,[],32,2);
    Screen(window,'BlendFunction',GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
    
    % STEP 3
    % Provide Eyelink with details about the graphics environment
    % and perform some initializations. The information is returned
    % in a structure that also contains useful defaults
    % and control codes (e.g. tracker state bit and Eyelink key values).
    el=EyelinkInitDefaults(window);
    
    % make sure that we get gaze data from the Eyelink
    Eyelink('Command', 'link_sample_data = LEFT,RIGHT,GAZE,AREA');
    
    [v, vs]=Eyelink('GetTrackerVersion');
    
    
    % open file to record data to
    edfFile=sprintf('%s%s',num2str(subjno),'P_',srand,'.edf'); % make sure the filename is no more than 8 character plus .EDF extension
    Eyelink('Openfile', edfFile);
    Eyelink('message', sprintf('SubjInfo: %s,%s,%s,%s,%s', 'DEMOStudyExperiment2', ' subjno', num2str(subjno), ' randnum',  srand)); %Puts info at top of EL file.
    
    % STEP 4
    % Calibrate the eye tracker
    EyelinkDoTrackerSetup(el);
    
end

%setup windows/drawings
%%%%

warning off MATLAB:DeprecatedLogicalAPI;
warning('off','MATLAB:dispatcher:InexactMatch');

%draw baby (off-screen) windows
port1=Screen(window,'OpenoffscreenWindow',white,[0 0 320 341]);
port2=Screen(window,'OpenoffscreenWindow',white,[0 0 320 341]);
port3=Screen(window,'OpenoffscreenWindow',white,[0 0 320 341]);
port4=Screen(window,'OpenoffscreenWindow',white,[0 0 320 341]);
blanktext=Screen(window,'OpenoffscreenWindow',white,[0 0 320 60]);
bigport=Screen(window,'OpenoffscreenWindow',white,[0 0 1280 1024]);
blank=Screen(window,'OpenoffscreenWindow',white,rect);

%set up variables in order to create a yellow box around the target
adjustment=[-5 -5 5 5];
yellow=[255 255 0];
yellowbox=Screen(window,'OpenoffscreenWindow',yellow,rect);

%on-screen port definitions
pt1=[Xqtr-160, Yqtr-170, Xqtr+160, Yqtr+170];
pt2=[Xqtr-160, Y3qtr-170,Xqtr+160, Y3qtr+170];
pt3=[X3qtr-160, Yqtr-170, X3qtr+160, Yqtr+170];
pt4=[X3qtr-160, Y3qtr-170, X3qtr+160, Y3qtr+170];

%reads in Cohorts list
PicListNumber = ['lists/items' num2str(listno) '.txt'];
fid = fopen(PicListNumber);
PicListbuffer = textscan(fid,'%s%s%s%s%s%s%s%s','HeaderLines',1); %trialID, listno, competitor, animacy, target picture, comp picture, other1 picture, other2 picture
fclose(fid);

%Opening txt file for results
%**********************************
filename=['output/Demo' num2str(subjno) '_' srand '.txt'];
fida=fopen(filename,'a');
fprintf(fida,'%s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\n','Trialnum','target', 'competitor', 'other1', 'other2', 'targetlocation','complocation','other1loc','other2loc','Target', 'ClickedPic','RT','CompetitorPresent','TargetAnimacy');

numtrials=(size(PicListbuffer{1},1));%total number of trials in the trial list
randomize=randperm(numtrials); %vector with the number of trials in a random order

%% Set Audio %%%
InitializePsychSound(0);
freq = 44100; % high frequency for high-quality audio recordings.
numchannels = 1;
buffersize=120;

audiochannel = PsychPortAudio('Open', [], 2, 1, freq, numchannels, buffersize);
%pahandle = PsychPortAudio('Open' [, deviceid][, mode][, reqlatencyclass][, freq][, channels][, buffersize][, suggestedLatency][, selectchannels][, specialFlags=0]);
% The first # (3) is for ASIO sound card.
% The second # is to 2 for recording mode; 1 is for playback, 3 is for
% BOTH. The second number =1 is to force it to do fast timing.

PsychPortAudio('GetAudioData', audiochannel, buffersize); % set aside from space for the sound that we're going to record
%[audiodata absrecposition overflow cstarttime] = PsychPortAudio('GetAudioData', pahandle [, amountToAllocateSecs][, minimumAmountToReturnSecs][, maximumAmountToReturnSecs][, singleType=0]);
% this sets aside x seconds of recording per trial better to error on
% the side of caution if you use MORE than the time you alloted, it starts writing over the start of the sound (like an old VHS tape
% this doesn't actually do any recording itself.  it just sets aside
% time for WHEN we do the recording


%begin trial loop
for currtrial = 1:numtrials; % for however many trials you list in your trial list
    SetMouse(Xcenter,Ycenter);    %sets mouse in center of screen at beginning of each trial.
    
    trialnum=randomize(currtrial); %selects the order of the current trial out of the random order vector.
    trialID=PicListbuffer{1}(trialnum); %the current trial ID (note this is the same as trialnum if you sorted the list by trial ID).
    comppres=PicListbuffer{3}(trialnum); %whether there's a competitor present or not.
    animacy=PicListbuffer{4}(trialnum); %whether the target is animate or not.
    target=['pictures/' char(PicListbuffer{5}(trialnum)) '.jpg'];
    competitor=['pictures/' char(PicListbuffer{6}(trialnum)) '.jpg'];
    other1=['pictures/' char(PicListbuffer{7}(trialnum)) '.jpg'];
    other2=['pictures/' char(PicListbuffer{8}(trialnum)) '.jpg'];
    sound=['sounds/' char(PicListbuffer{5}(trialnum)) '.wav']; %Soundfile for the trial
    
    %loads up pictures
    %%%
    pA=imread(target);
    pB=imread(competitor);
    pC=imread(other1);
    pD=imread(other2);
    blankpic=imread(('pictures/blank.jpg'));
    
    %copies pictures to baby windows
    Screen(port1,'PutImage',pA,[0 0 320 341],('srcCopy')); %put the target picture on offscreen "port1"
    Screen(port2,'PutImage',pB,[0 0 320 341],('srcCopy'));
    Screen(port3,'PutImage',pC,[0 0 320 341],('srcCopy'));
    Screen(port4,'PutImage',pD,[0 0 320 341],('srcCopy'));
    
    %create random ports
    randport=randperm(4);
    rpt1=eval(['pt' num2str(randport(1))]);
    rpt2=eval(['pt' num2str(randport(2))]);
    rpt3=eval(['pt' num2str(randport(3))]);
    rpt4=eval(['pt' num2str(randport(4))]);
    loc1=['pt',num2str(randport(1))]; %Location information to send to the edf file.
    loc2=['pt',num2str(randport(2))];
    loc3=['pt',num2str(randport(3))];
    loc4=['pt',num2str(randport(4))];
    
    if doeyetracking
        EyelinkDoDriftCorrection(el); %drift corrects in before all trials.
        
        % start recording eye position during picture preview.
        Eyelink('startrecording');
        eye_used = -1;    %what's this?
        %sends condition info to Eyetracker.
        %Note: the .asc conversion has a limit on how long this line can be.
        Eyelink('message', sprintf('Screen: %s,%s;%s,%s;%s,%s;%s,%s','target',loc1,'competitor',loc2,'o1',loc3,'o2', loc4)); % Prints to the .edf file information about object locations.
        %the stimuli message prints to the results file. I usually put my condition information here.
        Eyelink('message', sprintf('Stimuli: %s, %s, %s, %s, %s, %s', 'target animacy ', char(animacy), 'competitor present? ',char(comppres), 'trialID ', char(trialID)) ); %Prints to the .edf file information about condition
    end
    
    %% RECORD The AUDIO with fast latency..
    PsychPortAudio('Start',audiochannel); % this actually STARTS the recording ... just after that "stimuli" message shows in the ET data file.
    % we need to do the reverse of when we played a file
    % get the audio OUT of the buffer and into a matrix
    % then, save the matrix into a file
    
    %put pics on the on-screen window just after the audio starts recording so we make sure to get everything they might say.
    Screen('FillRect',window,white,rect); %blank out the screen.
    Screen('CopyWindow', yellowbox, window,rect,rpt1+adjustment); %put a yellow box at the place where the target picture will be pasted, to a screen location a tiny bit bigger than the space the target picture will occupy.
    Screen('CopyWindow', port1, window,[0 0 320 341],rpt1); %put offscreen port1 (which contains the target picture) onto random port1
    Screen('CopyWindow', port2, window,[0 0 320 341],rpt2);
    Screen('CopyWindow', port3, window,[0 0 320 341],rpt3);
    Screen('CopyWindow', port4, window,[0 0 320 341],rpt4);
    Screen('Flip',  window, [], 1); % don't erase!!!!
    
    RT1=GetSecs;
    
    %Click a picture to continue....
    selected=0;
    while selected==0
        [clickx,clicky,clicks]=GetMouse(window);
        if any(clicks)
            if clickx< ((rect(3))/2)
                if clicky< ((rect(4))/2)
                    selected=1;
                    wdclick='pt1';
                    RT2=GetSecs;
                elseif clicky>((rect(4))/2)
                    selected=2;
                    wdclick='pt2';
                    RT2=GetSecs;
                end
            else
                if clicky<((rect(4))/2)
                    selected=3;
                    wdclick='pt3';
                    RT2=GetSecs;
                else
                    selected=4;
                    wdclick='pt4';
                    RT2=GetSecs;
                end
            end
        end
    end
    
    
    RT=RT2-RT1;
    
    %**********************************
    %Writing Trial Results to txt file
    %**********************************
    fprintf(fida,'%s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %d\t %s\t %s\n',char(trialID),char(PicListbuffer{5}(trialnum)),char(PicListbuffer{6}(trialnum)),char(PicListbuffer{7}(trialnum)),char(PicListbuffer{8}(trialnum)),loc1,loc2,loc3,loc4,char(PicListbuffer{5}(trialnum)),wdclick,RT, char(comppres), char(animacy));
    
    WaitSecs(1);
    
    if doeyetracking
        Eyelink('stoprecording');
    end
    
    %clears screen for new trial
    
    Screen('CopyWindow',blank,window,rect,rect);
    Screen('Flip',  window, [], 1); % don't erase!!!!
    
    recordedaudio = PsychPortAudio('GetAudioData', audiochannel); % (at this point, since we've dumped things out of the buffer, we could record another 240 seconds if we wanted to)
    
    PsychPortAudio('Stop',audiochannel); % stop the recording channel
    % right now this is just a matrix in MATLAB.  we need to save it to a file on our hard drive
    filename = ['sounds/DemoE2_',num2str(currtrial),'_',num2str(subjno),'_',srand,'.wav']; %saves files to dir.
    wavwrite(recordedaudio, freq, filename);  %write the sound data RECORDEDAUDIO into a wav file with frequency FREQ, 3rd argument is the filename
    clear(filename);
    
end; %Ends trials loop.

%Save EDF to the subject PC too.
if doeyetracking
    status=Eyelink('closefile');
    if status ~=0
        fprintf('closefile error, status: %d',status)
    end
    status=Eyelink('ReceiveFile',edfFile,pwd,1);
    if status~=0
        fprintf('problem: ReceiveFile status: %d\n', status);
    end
    if 2==exist(edfFile, 'file')
        fprintf('Data file ''%s'' can be found in ''%s''\n', edfFile, pwd );
    else
        disp('unknown where data file went')
    end
end

if doeyetracking
    Eyelink('closefile');
    Eyelink('shutdown');
end

% --Thank You Screen --
textport=Screen(window,'OpenoffscreenWindow',white,rect);
Screen('DrawText', textport, 'You have finished the experiment. Thank you for participating! ', Xcenter-600, Ycenter+200, black, white, 0);
Screen('DrawText', textport, 'Please click to continue.  ', Xcenter-600, Ycenter+400, black, white, 0);
Screen('CopyWindow', textport, window, rect, rect);
Screen('Flip', window,  [],1);
GetClicks;

fclose(fida);
Screen('CloseAll');
clear all;